#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Data analyzation metrics

Each algorithm works on a set of handwritings. They have to be applied like
this:

 >>> import data_analyzation_metrics
 >>> a = [{'is_in_testset': 0,
           'formula_id': 31L,
           'handwriting': HandwrittenData(raw_data_id=2953),
           'formula_in_latex': 'A',
           'id': 2953L},
          {'is_in_testset': 0,
           'formula_id': 31L,
           'handwriting': HandwrittenData(raw_data_id=4037),
           'formula_in_latex': 'A',
           'id': 4037L},
          {'is_in_testset': 0,
           'formula_id': 31L,
           'handwriting': HandwrittenData(raw_data_id=4056),
           'formula_in_latex': 'A',
           'id': 4056L}]
 >>> creator_metric = Creator('creator.csv')
 >>> creator_metric(a)
"""
from __future__ import print_function
import os
import logging
import sys
import time
from collections import defaultdict
import math
import numpy

# hwrt modules
# HandwrittenData and preprocessing are needed because of pickle
from . import HandwrittenData  # pylint: disable=W0611
from . import preprocessing  # pylint: disable=W0611
from . import utils


def get_metrics(metrics_description):
    """Get metrics from a list of dictionaries. """
    return utils.get_objectlist(metrics_description,
                                config_key='data_analyzation_plugins',
                                module=sys.modules[__name__])

# Helper functions that are useful for some metrics


def prepare_file(filename):
    """Trunkate the file and return the filename."""
    directory = os.path.join(utils.get_project_root(), "analyzation/")
    if not os.path.exists(directory):
        os.makedirs(directory)
    workfilename = os.path.join(directory, filename)
    open(workfilename, 'w').close()  # Truncate the file
    return workfilename


def sort_by_formula_id(raw_datasets):
    """The parameter ``raw_datasets`` has to be of the format

        [{'is_in_testset': 0,
          'formula_id': 31L,
          'handwriting': HandwrittenData(raw_data_id=2953),
          'formula_in_latex': 'A',
          'id': 2953L},
         {'is_in_testset': 0,
          'formula_id': 31L,
          'handwriting': HandwrittenData(raw_data_id=4037),
          'formula_in_latex': 'A',
          'id': 4037L},
         {'is_in_testset': 0,
          'formula_id': 31L,
          'handwriting': HandwrittenData(raw_data_id=4056),
          'formula_in_latex': 'A',
          'id': 4056L}]
    """
    by_formula_id = defaultdict(list)
    for el in raw_datasets:
        by_formula_id[el['handwriting'].formula_id].append(el['handwriting'])
    return by_formula_id


# Only data analyzation calculation classes follow
# Every class must have a __str__, __repr__ and __call__ function where
# __call__ must take exactly one argument of type list of dictionaries
# Every class must have a constructor which takes the filename as a parameter.
# This filename has to be used to write the evaluation results
# (preferably in CSV format) to this file.
# prepare_file should be applied to every file in the constructor


class Creator(object):
    """Analyze who created most of the data."""

    def __init__(self, filename="creator.csv"):
        self.filename = prepare_file(filename)

    def __repr__(self):
        return "AnalyzeCreator(%s)" % self.filename

    def __str__(self):
        return "AnalyzeCreator(%s)" % self.filename

    def __call__(self, raw_datasets):
        write_file = open(self.filename, "a")
        write_file.write("creatorid,nr of recordings\n")  # heading

        print_data = defaultdict(int)
        start_time = time.time()
        for i, raw_dataset in enumerate(raw_datasets):
            if i % 100 == 0 and i > 0:
                utils.print_status(len(raw_datasets), i, start_time)
            print_data[raw_dataset['handwriting'].user_id] += 1
        print("\r100%"+"\033[K\n")

        # Sort the data by highest value, descending
        print_data = sorted(print_data.items(),
                            key=lambda n: n[1],
                            reverse=True)

        # Write data to file
        write_file.write("total,%i\n" %
                         sum([value for _, value in print_data]))
        for userid, value in print_data:
            write_file.write("%s,%i\n" % (str(userid), value))
        write_file.close()


class InstrokeSpeed(object):
    """Analyze how fast the points were in pixel/ms."""

    def __init__(self, filename="instroke_speed.csv"):
        self.filename = prepare_file(filename)

    def __repr__(self):
        return "InstrokeSpeed(%s)" % self.filename

    def __str__(self):
        return "InstrokeSpeed(%s)" % self.filename

    def __call__(self, raw_datasets):
        write_file = open(self.filename, "a")
        write_file.write("speed\n")  # heading

        print_data = []
        start_time = time.time()
        for i, raw_dataset in enumerate(raw_datasets):
            if i % 100 == 0 and i > 0:
                utils.print_status(len(raw_datasets), i, start_time)
            pointlist = raw_dataset['handwriting'].get_sorted_pointlist()

            for stroke in pointlist:
                for last_point, point in zip(stroke, stroke[1:]):
                    space_dist = math.hypot(last_point['x'] - point['x'],
                                            last_point['y'] - point['y'])
                    time_delta = point['time'] - last_point['time']
                    if time_delta == 0:
                        continue
                    print_data.append(space_dist/time_delta)
        print("\r100%"+"\033[K\n")
        # Sort the data by highest value, descending
        print_data = sorted(print_data, reverse=True)
        # Write data to file
        for value in print_data:
            write_file.write("%0.8f\n" % (value))

        logging.info("instroke speed mean: %0.8f", numpy.mean(print_data))
        logging.info("instroke speed std: %0.8f", numpy.std(print_data))
        write_file.close()


class InterStrokeDistance(object):
    """Analyze how much distance in px is between strokes."""

    def __init__(self, filename="dist_between_strokes.csv"):
        self.filename = prepare_file(filename)

    def __repr__(self):
        return "InterStrokeDistance(%s)" % self.filename

    def __str__(self):
        return "InterStrokeDistance(%s)" % self.filename

    def __call__(self, raw_datasets):
        write_file = open(self.filename, "a")
        write_file.write("speed\n")  # heading

        print_data = []
        start_time = time.time()
        for i, raw_dataset in enumerate(raw_datasets):
            if i % 100 == 0 and i > 0:
                utils.print_status(len(raw_datasets), i, start_time)
            pointlist = raw_dataset['handwriting'].get_sorted_pointlist()

            for last_stroke, stroke in zip(pointlist, pointlist[1:]):
                point1 = last_stroke[-1]
                point2 = stroke[0]
                space_dist = math.hypot(point1['x'] - point2['x'],
                                        point1['y'] - point2['y'])
                print_data.append(space_dist)
        print("\r100%"+"\033[K\n")
        # Sort the data by highest value, descending
        print_data = sorted(print_data, reverse=True)
        # Write data to file
        for value in print_data:
            write_file.write("%0.8f\n" % (value))

        logging.info("dist_between_strokes mean:\t%0.8fpx",
                     numpy.mean(print_data))
        logging.info("dist_between_strokes std: \t%0.8fpx",
                     numpy.std(print_data))
        write_file.close()


class TimeBetweenPointsAndStrokes(object):
    """For each recording: Store the average time between controll points of
       one stroke / controll points of two different strokes.
    """

    def __init__(self, filename="average_time_between_points.txt",
                 filename_strokes="average_time_between_strokes.txt"):
        self.filename_points = prepare_file(filename)
        self.filename_strokes = prepare_file(filename_strokes)

    def __repr__(self):
        return "TimeBetweenPointsAndStrokes(%s, %s)" % \
            (self.filename_points, self.filename_strokes)

    def __str__(self):
        return "TimeBetweenPointsAndStrokes(%s, %s)" % \
            (self.filename_points, self.filename_strokes)

    def __call__(self, raw_datasets):
        average_between_points = open(self.filename_points, "a")
        average_between_strokes = open(self.filename_strokes, "a")
        start_time = time.time()
        for i, raw_dataset in enumerate(raw_datasets):
            if i % 100 == 0 and i > 0:
                utils.print_status(len(raw_datasets), i, start_time)

            # Do the work
            times_between_points, times_between_strokes = [], []
            last_stroke_end = None
            for stroke in raw_dataset['handwriting'].get_sorted_pointlist():
                if last_stroke_end is not None:
                    times_between_strokes.append(stroke[-1]['time'] -
                                                 last_stroke_end)
                last_stroke_end = stroke[-1]['time']
                for point1, point2 in zip(stroke, stroke[1:]):
                    delta = point2['time'] - point1['time']
                    times_between_points.append(delta)
            # The recording might only have one point
            if len(times_between_points) > 0:
                tmp = times_between_points
                average_between_points.write("%0.2f\n" % numpy.average(tmp))
            # The recording might only have one stroke
            if len(times_between_strokes) > 0:
                tmp = times_between_strokes
                average_between_strokes.write("%0.2f\n" % numpy.average(tmp))
        print("\r100%"+"\033[K\n")
        average_between_points.close()
        average_between_strokes.close()


class AnalyzeErrors(object):
    """Analyze the number of errors in the dataset."""

    def __init__(self, filename="errors.txt", time_max_threshold=30*1000):
        self.filename = prepare_file(filename)
        self.time_max_threshold = time_max_threshold  # in ms
        self.dot_symbols = ['i', 'j', '\cdot', '\div', '\\because',
                            '\\therefore']

    def __repr__(self):
        return "AnalyzeErrors"

    def __str__(self):
        return "AnalyzeErrors"

    def _write_data(self, symbols, err_recs, nr_recordings,
                    total_error_count, percentages, time_max_list):
        """Write all obtained data to a file.

        :param symbols: List of all symbols with the count of recordings
        :type symbols: List of tuples (String, non-negative int)
        :param err_recs: count of recordings by error type
        :type err_recs: dictionary
        :param nr_recordings: number of recordings
        :type nr_recordings: non-negative int
        :param total_error_count: Count of all error that have happened by type
        :type total_error_count: dictionary
        :param percentages: List of all recordings where removing the dots
                            changed the size of the bounding box.
        :param time_max_list: List of all recordings where the recording time
                              is above a threshold."""
        write_file = open(self.filename, "a")
        s = ""
        for symbol, count in sorted(symbols.items(), key=lambda n: n[0]):
            if symbol in ['a', '0', 'A']:
                s += "\n%s (%i), " % (symbol, count)
            elif symbol in ['z', '9', 'Z']:
                s += "%s (%i) \n" % (symbol, count)
            else:
                s += "%s (%i), " % (symbol, count)
        print("## Data", file=write_file)
        print("Symbols: %i" % len(symbols), file=write_file)
        print("Recordings: %i" % sum(symbols.values()), file=write_file)
        print("```", file=write_file)
        print(s[:-1], file=write_file)
        print("```", file=write_file)

        # Show errors
        print("Recordings with wild points: %i (%0.2f%%)" %
              (err_recs['wild_points'],
               float(err_recs['wild_points'])/nr_recordings*100),
              file=write_file)
        print("wild points: %i" % total_error_count['wild_points'],
              file=write_file)
        print("Recordings with missing stroke: %i (%0.2f%%)" %
              (err_recs['missing_stroke'],
               float(err_recs['missing_stroke'])/nr_recordings*100),
              file=write_file)
        print("Recordings with errors: %i (%0.2f%%)" %
              (err_recs['total'],
               float(err_recs['total'])/nr_recordings*100),
              file=write_file)
        print("Recordings with dots: %i (%0.2f%%)" %
              (err_recs['single_dots'],
               float(err_recs['single_dots'])/nr_recordings*100),
              file=write_file)
        print("dots: %i" % total_error_count['single_dots'], file=write_file)
        print("size changing removal: %i (%0.2f%%)" %
              (len(percentages),
               float(len(percentages))/nr_recordings*100),
              file=write_file)
        print("%i recordings took more than %i ms. That were: " %
              (len(time_max_list), self.time_max_threshold),
              file=write_file)
        for recording in time_max_list:
            print("* %ims: %s: %s" %
                  (recording.get_time(),
                   utils.get_readable_time(recording.get_time()),
                   recording),
                  file=write_file)
        write_file.close()

    def __call__(self, raw_datasets):
        # Initialize variables
        symbols = defaultdict(int)

        # Count errornous recordings
        err_recs = {'wild_points': 0, 'missing_stroke': 0,
                    'single_dots': 0,  # except symbols_with_dots
                    'total': 0}

        # Count errors (one type of error might occur multiple times in
        # a single recording)
        total_error_count = {'wild_points': 0, 'single_dots': 0}

        percentages = []

        # List with recordings that are over the time maximum
        time_max_list = []

        for raw_dataset in raw_datasets:
            recording = raw_dataset['handwriting']
            symbols[recording.formula_in_latex] += 1
            if recording.get_time() > self.time_max_threshold:
                time_max_list.append(recording)
            if recording.wild_point_count > 0:
                err_recs['wild_points'] += 1
                total_error_count['wild_points'] += recording.wild_point_count
            err_recs['missing_stroke'] += recording.missing_stroke
            if recording.wild_point_count > 0 or \
               recording.missing_stroke:
                err_recs['total'] += 1
            if recording.count_single_dots() > 0 and \
               raw_dataset['formula_in_latex'] not in self.dot_symbols and \
               "dots" not in raw_dataset['formula_in_latex']:
                err_recs['single_dots'] += 1
                old_area = recording.get_area()
                tmp = [preprocessing.RemoveDots()]
                recording.preprocessing(tmp)
                new_area = recording.get_area()
                percentage = float(new_area)/float(old_area)
                if percentage < 1.0:
                    percentages.append(percentage)
            total_error_count['single_dots'] += recording.count_single_dots()

        time_max_list = sorted(time_max_list,
                               key=lambda n: n.get_time(),
                               reverse=True)

        self._write_data(symbols, err_recs, len(raw_datasets),
                         total_error_count, percentages,
                         time_max_list)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Preprocessing algorithms.

Each algorithm works on the HandwrittenData class. They have to be applied like
this:

    >>> a = HandwrittenData(...)
    >>> preprocessing_queue = [ScaleAndShift(),
                               StrokeConnect(),
                               DouglasPeucker(epsilon=0.2),
                               SpaceEvenly(number=100)]
    >>> a.preprocessing(preprocessing_queue)
"""

import logging
import sys
import numpy
from scipy.interpolate import interp1d
import math

# hwrt modules
from . import HandwrittenData
from . import utils
from . import geometry


def euclidean_distance(p1, p2):
    """Calculate the euclidean distance of two 2D points.

    >>> euclidean_distance({'x': 0, 'y': 0}, {'x': 0, 'y': 3})
    3.0
    >>> euclidean_distance({'x': 0, 'y': 0}, {'x': 0, 'y': -3})
    3.0
    >>> euclidean_distance({'x': 0, 'y': 0}, {'x': 3, 'y': 4})
    5.0
    """
    return math.sqrt((p1["x"]-p2["x"])**2 + (p1["y"]-p2["y"])**2)


def get_preprocessing_queue(preprocessing_list):
    """Get preprocessing queue from a list of dictionaries

    >>> l = [{'RemoveDuplicateTime': None},
             {'ScaleAndShift': [{'center': True}]}
            ]
    >>> get_preprocessing_queue(l)
    [RemoveDuplicateTime, ScaleAndShift
     - center: True
     - max_width: 1
     - max_height: 1
    ]
    """
    return utils.get_objectlist(preprocessing_list,
                                config_key='preprocessing',
                                module=sys.modules[__name__])


def print_preprocessing_list(preprocessing_queue):
    """Print the ``preproc_list`` in a human-readable form.

    :param preprocessing_queue: Algorithms that get applied for preprocessing.
    :type preprocessing_queue: List of preprocessing objects.
    """
    print("## Preprocessing")
    print("```")
    for algorithm in preprocessing_queue:
        print("* " + str(algorithm))
    print("```")

# Only preprocessing classes follow
# Everyone must have a __str__, __repr__ and __call__
# where
# * __call__ must take exactly one argument of type HandwrittenData
# * __call__ must call the Handwriting.set_points


class RemoveDuplicateTime(object):
    """If a recording has two points with the same timestamp, than the second
       point will be discarded. This is useful for a couple of algorithms that
       don't expect two points at the same time."""
    def __repr__(self):
        return "RemoveDuplicateTime"

    def __str__(self):
        return "remove duplicate time"

    def __call__(self, handwritten_data):
        assert isinstance(handwritten_data, HandwrittenData.HandwrittenData), \
            "handwritten data is not of type HandwrittenData, but of %r" % \
            type(handwritten_data)
        pointlist = handwritten_data.get_pointlist()
        new_pointlist = []
        times = []
        for stroke in pointlist:
            new_stroke = []
            for point in stroke:
                if point['time'] not in times:
                    new_stroke.append(point)
                    times.append(point['time'])
            if len(new_stroke) > 0:
                new_pointlist.append(new_stroke)
        # Make sure there are no duplicates
        times = [point['time'] for stroke in new_pointlist for point in stroke]
        assert len(times) == len(set(times)), \
            ("The list of all times in RemoveDuplicateTime has %i values, "
             "but the set has %i values: %s --- %s") % \
            (len(times), len(set(times)), pointlist, new_pointlist)
        handwritten_data.set_pointlist(new_pointlist)


class RemoveDots(object):
    """Remove all strokes that have only a single point (a dot) from the
       recording, except if the whole recording consists of dots only.
    """
    def __repr__(self):
        return "Remove_points"

    def __str__(self):
        return "remove points"

    def __call__(self, handwritten_data):
        assert isinstance(handwritten_data, HandwrittenData.HandwrittenData), \
            "handwritten data is not of type HandwrittenData, but of %r" % \
            type(handwritten_data)
        pointlist = handwritten_data.get_pointlist()
        has_nonpoint_stroke = False
        # Check if recording has non-point stroke:
        for stroke in pointlist:
            if len(stroke) > 1:
                has_nonpoint_stroke = True
        if has_nonpoint_stroke:
            new_pointlist = []
            for stroke in pointlist:
                if len(stroke) > 1:
                    new_pointlist.append(stroke)
            handwritten_data.set_pointlist(new_pointlist)


class ScaleAndShift(object):
    """ Scale a recording so that it fits into a unit square. This keeps the
        aspect ratio. Then the recording is shifted. The default way is to
        shift it so that the recording is in [0, 1] × [0,1]. However, it
        can also be used to be centered within [-1, 1] × [-1, 1] around the
        origin (0, 0) by setting center=True and center_other=True.
    """
    def __init__(self, center=False, max_width=1., max_height=1.,
                 width_add=0, height_add=0, center_other=False):
        self.center = center
        self.max_width = max_width
        self.max_height = max_height
        self.width_add = width_add
        self.height_add = height_add
        self.center_other = center_other

    def __repr__(self):
        return ("ScaleAndShift\n"
                " - center: %r\n"
                " - max_width: %i\n"
                " - max_height: %i\n") % \
            (self.center, self.max_width, self.max_height)

    def __str__(self):
        return ("Scale and shift\n"
                " - center: %r\n"
                " - max_width: %i\n"
                " - max_height: %i\n") % \
            (self.center, self.max_width, self.max_height)

    def _get_parameters(self, handwritten_data):
        """ Take a list of points and calculate the factors for scaling and
            moving it so that it's in the unit square. Keept the aspect
            ratio.
            Optionally center the points inside of the unit square.
        """
        a = handwritten_data.get_bounding_box()

        width = a['maxx'] - a['minx'] + self.width_add
        height = a['maxy'] - a['miny'] + self.height_add

        factor_x, factor_y = 1, 1
        if width != 0:
            factor_x = self.max_width/width

        if height != 0:
            factor_y = self.max_height/height

        factor = min(factor_x, factor_y)
        addx, addy = 0, 0

        if self.center:
            # Only one dimension (x or y) has to be centered (the smaller one)
            add = -(factor/(2*max(factor_x, factor_y)))

            if factor == factor_x:
                addy = add
                if self.center_other:
                    addx = -(width*factor/2.0)
            else:
                addx = add
                if self.center_other:
                    addy = -(height*factor/2.0)

        return {"factor": factor, "addx": addx, "addy": addy,
                "minx": a['minx'], "miny": a['miny'], "mint": a['mint']}

    def __call__(self, handwritten_data):
        assert isinstance(handwritten_data, HandwrittenData.HandwrittenData), \
            "handwritten data is not of type HandwrittenData, but of %r" % \
            type(handwritten_data)

        tmp = self._get_parameters(handwritten_data)
        factor, addx, addy = tmp['factor'], tmp['addx'], tmp['addy']
        minx, miny, mint = tmp['minx'], tmp['miny'], tmp['mint']

        pointlist = handwritten_data.get_pointlist()
        for strokenr, stroke in enumerate(pointlist):
            for key, p in enumerate(stroke):
                pointlist[strokenr][key] = {
                    "x": (p["x"] - minx) * factor + addx,
                    "y": (p["y"] - miny) * factor + addy,
                    "time": p["time"] - mint}
                if "pen_down" in p:
                    pointlist[strokenr][key]["pen_down"] = p["pen_down"]
        handwritten_data.set_pointlist(pointlist)
        assert self.max_width - handwritten_data.get_width() >= -0.00001, \
            "max_width: %0.5f; width: %0.5f" % (self.max_width,
                                                handwritten_data.get_width())
        assert self.max_height - handwritten_data.get_height() >= -0.00001, \
            "max_height: %0.5f; height: %0.5f" % \
            (self.max_height, handwritten_data.get_height())


class SpaceEvenly(object):
    """Space the points evenly in time over the complete recording. The
       parameter 'number' defines how many."""
    def __init__(self, number=100, kind='cubic'):
        self.number = number
        self.kind = kind

    def __repr__(self):
        return ("SpaceEvenly\n"
                " - number: %i\n"
                " - kind: %s\n") % \
            (self.number, self.kind)

    def __str__(self):
        return ("Space evenly\n"
                " - number: %i\n"
                " - kind: %s\n") % \
            (self.number, self.kind)

    def _calculate_pen_down_strokes(self, pointlist, times=None):
        """Calculate the intervall borders 'times' that contain the information
           when a stroke started, when it ended and how it should be
           interpolated."""
        if times is None:
            times = []
        for stroke in pointlist:
            stroke_info = {"start": stroke[0]['time'],
                           "end": stroke[-1]['time'],
                           "pen_down": True}
            # set up variables for interpolation
            x, y, t = [], [], []
            for point in stroke:
                if point['time'] not in t:
                    x.append(point['x'])
                    y.append(point['y'])
                    t.append(point['time'])
            x, y = numpy.array(x), numpy.array(y)
            if len(t) == 1:
                # constant interpolation
                fx, fy = lambda x: float(x), lambda y: float(y)
            elif len(t) == 2:
                # linear interpolation
                fx, fy = interp1d(t, x, 'linear'), interp1d(t, y, 'linear')
            elif len(t) == 3:
                # quadratic interpolation
                fx = interp1d(t, x, 'quadratic')
                fy = interp1d(t, y, 'quadratic')
            else:
                fx, fy = interp1d(t, x, self.kind), interp1d(t, y, self.kind)
            stroke_info['fx'] = fx
            stroke_info['fy'] = fy
            times.append(stroke_info)
        return times

    def _calculate_pen_up_strokes(self, pointlist, times=None):
        """ 'Pen-up' strokes are virtual strokes that were not drawn. It
            models the time when the user moved from one stroke to the next.
        """
        if times is None:
            times = []
        for i in range(len(pointlist) - 1):
            stroke_info = {"start": pointlist[i][-1]['time'],
                           "end": pointlist[i+1][0]['time'],
                           "pen_down": False}
            x, y, t = [], [], []
            for point in [pointlist[i][-1], pointlist[i+1][0]]:
                if point['time'] not in t:
                    x.append(point['x'])
                    y.append(point['y'])
                    t.append(point['time'])
            if len(x) == 1:
                # constant interpolation
                fx, fy = lambda x: float(x), lambda y: float(y)
            else:
                # linear interpolation
                x, y = numpy.array(x), numpy.array(y)
                fx = interp1d(t, x, kind='linear')
                fy = interp1d(t, y, kind='linear')
            stroke_info['fx'] = fx
            stroke_info['fy'] = fy
            times.append(stroke_info)
        return times

    def __call__(self, handwritten_data):
        assert isinstance(handwritten_data, HandwrittenData.HandwrittenData), \
            "handwritten data is not of type HandwrittenData, but of %r" % \
            type(handwritten_data)
        # Make sure that the lists are sorted
        pointlist = handwritten_data.get_sorted_pointlist()

        # Build 'times' datastructure which will contain information about
        # strokes and when they were started / ended and how they should be
        # interpolated
        times = self._calculate_pen_down_strokes(pointlist)
        times = self._calculate_pen_up_strokes(pointlist, times)

        tnew = numpy.linspace(pointlist[0][0]['time'],
                              pointlist[-1][-1]['time'],
                              self.number)

        # Create the new pointlist
        new_pointlist = []
        for time in tnew:
            for stroke_interval in times:
                if stroke_interval["start"] <= time <= stroke_interval["end"]:
                    x = float(stroke_interval['fx'](time))
                    y = float(stroke_interval['fy'](time))
                    time = float(time)
                    new_pointlist.append({'x': x, 'y': y, 'time': time,
                                          'pen_down':
                                          stroke_interval['pen_down']})
        handwritten_data.set_pointlist([new_pointlist])


class SpaceEvenlyPerStroke(object):
    """Space the points evenly for every single stroke separately. The
       parameter `number` defines how many points are used per stroke and the
       parameter `kind` defines which kind of interpolation is used. Possible
       values include `cubic`, `quadratic`, `linear`, `nearest`. This part of
       the implementation relies on
       :mod:`scipy.interpolate.interp1d <scipy:scipy.interpolate.interp1d>`.
    """
    def __init__(self, number=100, kind='cubic'):
        self.number = number
        self.kind = kind

    def __repr__(self):
        return ("SpaceEvenlyPerStroke\n"
                " - number: %i\n"
                " - kind: %s\n") % \
            (self.number, self.kind)

    def __str__(self):
        return ("Space evenly per stroke\n"
                " - number: %i\n"
                " - kind: %s\n") % \
            (self.number, self.kind)

    def _space(self, handwritten_data, stroke, kind):
        """Do the interpolation of 'kind' for 'stroke'"""
        new_stroke = []
        stroke = sorted(stroke, key=lambda p: p['time'])

        x, y, t = [], [], []

        for point in stroke:
            x.append(point['x'])
            y.append(point['y'])
            t.append(point['time'])

        x, y = numpy.array(x), numpy.array(y)
        failed = False
        try:
            fx = interp1d(t, x, kind=kind)
            fy = interp1d(t, y, kind=kind)
        except Exception as e:  # pylint: disable=W0703
            if handwritten_data.raw_data_id is not None:
                logging.debug("spline failed for raw_data_id %i",
                              handwritten_data.raw_data_id)
            else:
                logging.debug("spline failed")
            logging.debug(e)
            failed = True

        tnew = numpy.linspace(t[0], t[-1], self.number)

        # linear interpolation fallback due to
        # https://github.com/scipy/scipy/issues/3868
        if failed:
            try:
                fx = interp1d(t, x, kind='linear')
                fy = interp1d(t, y, kind='linear')
                failed = False
            except Exception as e:
                logging.debug("len(stroke) = %i", len(stroke))
                logging.debug("len(x) = %i", len(x))
                logging.debug("len(y) = %i", len(y))
                logging.debug("stroke=%s", stroke)
                raise e

        for x, y, t in zip(fx(tnew), fy(tnew), tnew):
            new_stroke.append({'x': x, 'y': y, 'time': t})
        return new_stroke

    def __call__(self, handwritten_data):
        assert isinstance(handwritten_data, HandwrittenData.HandwrittenData), \
            "handwritten data is not of type HandwrittenData, but of %r" % \
            type(handwritten_data)
        pointlist = handwritten_data.get_sorted_pointlist()
        new_pointlist = []

        for stroke in pointlist:
            if len(stroke) < 2:
                # Don't do anything if there are less than 2 points
                new_stroke = stroke
            elif 2 <= len(stroke) <= 3:
                # Linear interpolation for 2 or 3 points
                new_stroke = self._space(handwritten_data,
                                         stroke,
                                         'linear')
            else:
                new_stroke = self._space(handwritten_data,
                                         stroke,
                                         self.kind)
            new_pointlist.append(new_stroke)
        handwritten_data.set_pointlist(new_pointlist)


class DouglasPeucker(object):
    """Apply the Douglas-Peucker stroke simplification algorithm separately to
       each stroke of the recording. The algorithm has a threshold parameter
       `epsilon` that indicates how much the stroke is simplified. The smaller
       the parameter, the closer will the resulting strokes be to the original.
    """
    def __init__(self, epsilon=0.2):
        self.epsilon = epsilon

    def __repr__(self):
        return "DouglasPeucker (epsilon: %0.2f)\n" % self.epsilon

    def __str__(self):
        return "DouglasPeucker (epsilon: %0.2f)\n" % self.epsilon

    def _stroke_simplification(self, pointlist):
        """The Douglas-Peucker line simplification takes a list of points as an
           argument. It tries to simplifiy this list by removing as many points
           as possible while still maintaining the overall shape of the stroke.
           It does so by taking the first and the last point, connecting them
           by a straight line and searchin for the point with the highest
           distance. If that distance is bigger than 'epsilon', the point is
           important and the algorithm continues recursively."""

        # Find the point with the biggest distance
        dmax = 0
        index = 0
        for i in range(1, len(pointlist)):
            d = geometry.perpendicular_distance(pointlist[i],
                                                pointlist[0],
                                                pointlist[-1])
            if d > dmax:
                index = i
                dmax = d

        # If the maximum distance is bigger than the threshold 'epsilon', then
        # simplify the pointlist recursively
        if dmax >= self.epsilon:
            # Recursive call
            rec_results1 = self._stroke_simplification(pointlist[0:index])
            rec_results2 = self._stroke_simplification(pointlist[index:])
            result_list = rec_results1[:-1] + rec_results2
        else:
            result_list = [pointlist[0], pointlist[-1]]

        return result_list

    def __call__(self, handwritten_data):
        assert isinstance(handwritten_data, HandwrittenData.HandwrittenData), \
            "handwritten data is not of type HandwrittenData, but of %r" % \
            type(handwritten_data)
        pointlist = handwritten_data.get_pointlist()

        for i in range(0, len(pointlist)):
            pointlist[i] = self._stroke_simplification(pointlist[i])
        handwritten_data.set_pointlist(pointlist)
        # This might have duplicated points! Filter them!
        handwritten_data.preprocessing([RemoveDuplicateTime()])


class StrokeConnect(object):
    """`StrokeConnect`: Detect if strokes were probably accidentally
       disconnected. If that is the case, connect them. This is detected by the
       threshold parameter `minimum_distance`. If the distance between the end
       point of a stroke and the first point of the next stroke is below the
       minimum distance, the strokes will be connected.
    """
    def __init__(self, minimum_distance=0.05):
        self.minimum_distance = minimum_distance

    def __repr__(self):
        return "StrokeConnect (minimum_distance: %0.2f)" % \
            self.minimum_distance

    def __str__(self):
        return "Stroke connect (minimum_distance: %0.2f)" % \
            self.minimum_distance

    def __call__(self, handwritten_data):
        assert isinstance(handwritten_data, HandwrittenData.HandwrittenData), \
            "handwritten data is not of type HandwrittenData, but of %r" % \
            type(handwritten_data)
        pointlist = handwritten_data.get_pointlist()

        # Connecting strokes makes only sense when there are multiple strokes
        if len(pointlist) > 1:
            strokes = []
            last_appended = False
            i = 0
            while i < len(pointlist)-1:
                last_point = pointlist[i][-1]
                first_point = pointlist[i+1][0]
                if euclidean_distance(last_point, first_point) < \
                   self.minimum_distance:
                    strokes.append(pointlist[i]+pointlist[i+1])
                    pointlist[i+1] = strokes[-1]
                    if i == len(pointlist)-2:
                        last_appended = True
                    i += 1
                else:
                    strokes.append(pointlist[i])
                i += 1
            if not last_appended:
                strokes.append(pointlist[-1])
            handwritten_data.set_pointlist(strokes)


class DotReduction(object):
    """Reduce strokes where the maximum distance between points is below a
       `threshold` to a single dot.
    """
    def __init__(self, threshold=5):
        self.threshold = threshold

    def __repr__(self):
        return "DotReduction (threshold: %0.2f)" % \
            self.threshold

    def __str__(self):
        return "DotReduction (threshold: %0.2f)" % \
            self.threshold

    def __call__(self, handwritten_data):
        assert isinstance(handwritten_data, HandwrittenData.HandwrittenData), \
            "handwritten data is not of type HandwrittenData, but of %r" % \
            type(handwritten_data)

        def _get_max_distance(L):
            """Find the maximum distance between two points in a list of points
            :param L: list of points
            :type L: list
            :returns: maximum distance bewtween two points
            :rtype: float
            """
            if len(L) <= 1:
                return -1
            else:
                max_dist = euclidean_distance(L[0], L[1])
                for i in range(len(L)-1):
                    for j in range(i+1, len(L)):
                        max_dist = max(euclidean_distance(L[i], L[j]),
                                       max_dist)
                return max_dist

        def _get_average_point(pointlist):
            """Calculate the average point.
            :param pointlist: list of points
            :type pointlist: list
            :returns: a single point
            :rtype: dict
            """
            x, y, t = 0, 0, 0
            for point in pointlist:
                x += point['x']
                y += point['y']
                t += point['time']
            x = float(x) / len(pointlist)
            y = float(y) / len(pointlist)
            t = float(t) / len(pointlist)
            return {'x': x, 'y': y, 'time': t}

        new_pointlist = []
        pointlist = handwritten_data.get_pointlist()
        for stroke in pointlist:
            new_stroke = stroke
            if len(stroke) > 1 and _get_max_distance(stroke) < self.threshold:
                new_stroke = [_get_average_point(stroke)]
            new_pointlist.append(new_stroke)

        handwritten_data.set_pointlist(new_pointlist)


class WildPointFilter(object):
    """Find wild points and remove them. The threshold means
       speed in pixels / ms.
    """
    def __init__(self, threshold=3.0):
        """The threshold is a speed threshold"""
        self.threshold = threshold

    def __repr__(self):
        return "WildPointFilter"

    def __str__(self):
        return "Wild point filter (threshold: %0.2f)" % \
            self.threshold

    def __call__(self, handwritten_data):
        assert isinstance(handwritten_data, HandwrittenData.HandwrittenData), \
            "handwritten data is not of type HandwrittenData, but of %r" % \
            type(handwritten_data)
        new_pointlist = []
        pointlist = handwritten_data.get_sorted_pointlist()
        for stroke in pointlist:
            new_stroke = []
            for last_point, point in zip(stroke, stroke[1:]):
                space_dist = math.hypot(last_point['x'] - point['x'],
                                        last_point['y'] - point['y'])
                time_dist = float(point['time'] - last_point['time'])
                if time_dist == 0:
                    continue
                speed = space_dist/time_dist
                if speed < self.threshold:
                    new_stroke.append(point)

            new_pointlist.append(new_stroke)
        # Bounding box criterion:
        # If the distance from point to all others strokes bounding boxes is
        # more than 1/5 of the whole size, it is a wild point


class WeightedAverageSmoothing(object):
    """Smooth every stroke by a weighted average. This algorithm takes a list
       `theta` of 3 numbers that are the weights used for smoothing."""
    def __init__(self, theta=None):
        """Theta is a list of 3 non-negative numbers"""
        if theta is None:
            theta = [1./6, 4./6, 1./6]
        assert len(theta) == 3, \
            "theta has length %i, but should have length 3" % \
            len(theta)
        theta = [float(element) for element in theta]

        # Normalize parameters to a sum of 1
        self.theta = list(1./sum(theta) * numpy.array(theta))

    def __repr__(self):
        return "WeightedAverageSmoothing"

    def __str__(self):
        return "Weighted average smoothing (theta: %s)" % \
            self.theta

    def _calculate_average(self, points):
        """Calculate the arithmetic mean of the points x and y coordinates
           seperately.
        """
        assert len(self.theta) == len(points), \
            "points has length %i, but should have length %i" % \
            (len(points), len(self.theta))
        new_point = {'x': 0, 'y': 0, 'time': 0}
        for key in new_point:
            new_point[key] = self.theta[0] * points[0][key] + \
                self.theta[1] * points[1][key] + \
                self.theta[2] * points[2][key]
        return new_point

    def __call__(self, handwritten_data):
        assert isinstance(handwritten_data, HandwrittenData.HandwrittenData), \
            "handwritten data is not of type HandwrittenData, but of %r" % \
            type(handwritten_data)
        new_pointlist = []
        pointlist = handwritten_data.get_sorted_pointlist()
        for stroke in pointlist:
            tmp = [stroke[0]]
            new_pointlist.append(tmp)
            if len(stroke) > 1:
                for i in range(1, len(stroke)-1):
                    points = [stroke[i-1], stroke[i], stroke[i+1]]
                    p = self._calculate_average(points)
                    new_pointlist[-1].append(p)
                new_pointlist[-1].append(stroke[-1])
        handwritten_data.set_pointlist(new_pointlist)

if __name__ == '__main__':
    import doctest
    doctest.testmod()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Analyze data in a pickle file by maximum time / width / height and
   similar features.
"""

from __future__ import print_function
import os
import logging
import sys
try:  # Python 2
    import cPickle as pickle
except ImportError:  # Python 3
    import pickle
import numpy

# hwrt modules
# HandwrittenData is necessary because of pickle
from hwrt import HandwrittenData
sys.modules['HandwrittenData'] = HandwrittenData
import hwrt.features as features
import hwrt.utils as utils
import hwrt.data_analyzation_metrics as dam


def filter_label(label, replace_by_similar=True):
    """Some labels currently don't work together because of LaTeX naming
       clashes. Those will be replaced by simple strings. """
    bad_names = ['celsius', 'degree', 'ohm', 'venus', 'mars', 'astrosun',
                 'fullmoon', 'leftmoon', 'female', 'male', 'checked',
                 'diameter', 'sun', 'Bowtie', 'sqrt',
                 'cong', 'copyright', 'dag', 'parr', 'notin', 'dotsc',
                 'mathds', 'mathfrak']
    if any(label[1:].startswith(bad) for bad in bad_names):
        if label == '\\dag' and replace_by_similar:
            return '\\dagger'
        elif label == '\\diameter' and replace_by_similar:
            return '\\O'
        return label[1:]
    else:
        return label


def analyze_feature(raw_datasets, feature, basename="aspect_ratios"):
    """Apply ``feature`` to all recordings in ``raw_datasets``. Store the
       results in two files. One file stores the raw result, the other one
       groups the results by symbols and stores the mean, standard deviation
       and the name of the symbol as a csv file.
    """

    # Prepare files
    csv_file = dam.prepare_file(basename+'.csv')
    raw_file = dam.prepare_file(basename+'.raw')

    csv_file = open(csv_file, 'a')
    raw_file = open(raw_file, 'a')

    csv_file.write("label,mean,std\n")  # Write header
    raw_file.write("latex,raw_data_id,value\n")  # Write header

    print_data = []
    for _, datasets in dam.sort_by_formula_id(raw_datasets).items():
        values = []
        for data in datasets:
            value = feature(data)[0]
            values.append(value)
            raw_file.write("%s,%i,%0.2f\n" % (datasets[0].formula_in_latex,
                                              data.raw_data_id,
                                              value))
        label = filter_label(datasets[0].formula_in_latex)
        print_data.append((label, numpy.mean(values), numpy.std(values)))

    # Sort the data by highest mean, descending
    print_data = sorted(print_data, key=lambda n: n[1], reverse=True)

    # Write data to file
    for label, mean, std in print_data:
        csv_file.write("%s,%0.2f,%0.2f\n" % (label, mean, std))
    csv_file.close()


def main(handwriting_datasets_file, analyze_features):
    """Start the creation of the wanted metric."""
    # Load from pickled file
    logging.info("Start loading data '%s' ...", handwriting_datasets_file)
    loaded = pickle.load(open(handwriting_datasets_file))
    raw_datasets = loaded['handwriting_datasets']
    logging.info("%i datasets loaded.", len(raw_datasets))
    logging.info("Start analyzing...")

    if analyze_features:
        featurelist = [(features.AspectRatio(), "aspect_ratio.csv"),
                       (features.ReCurvature(1), "re_curvature.csv"),
                       (features.Height(), "height.csv"),
                       (features.Width(), "width.csv"),
                       (features.Time(), "time.csv"),
                       (features.Ink(), "ink.csv"),
                       (features.StrokeCount(), "stroke-count.csv")]
        for feat, filename in featurelist:
            logging.info("create %s...", filename)
            analyze_feature(raw_datasets, feat, filename)

    # Analyze everything specified in configuration
    cfg = utils.get_project_configuration()
    if 'data_analyzation_queue' in cfg:
        metrics = dam.get_metrics(cfg['data_analyzation_queue'])
        for metric in metrics:
            logging.info("Start metric %s...", str(metric))
            metric(raw_datasets)
    else:
        logging.info("No 'data_analyzation_queue' in ~/.hwrtrc")


def get_parser():
    """Return the parser object for this script."""
    project_root = utils.get_project_root()

    # Get latest (raw) dataset
    dataset_folder = os.path.join(project_root, "raw-datasets")
    latest_dataset = utils.get_latest_in_folder(dataset_folder, "raw.pickle")

    from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
    parser = ArgumentParser(description=__doc__,
                            formatter_class=ArgumentDefaultsHelpFormatter)
    parser.add_argument("-d", "--handwriting_datasets",
                        dest="handwriting_datasets",
                        help="where are the pickled handwriting_datasets?",
                        metavar="FILE",
                        type=lambda x: utils.is_valid_file(parser, x),
                        default=latest_dataset)
    parser.add_argument("-f", "--features",
                        dest="analyze_features",
                        help="analyze features",
                        action="store_true",
                        default=False)
    return parser


if __name__ == '__main__':
    args = get_parser().parse_args()
    main(args.handwriting_datasets, args.analyze_features)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Start a webserver which can record the data and work as a classifier."""

import pkg_resources
from flask import Flask, request, render_template
from flask_bootstrap import Bootstrap
import os
import json
import requests
import logging
import urllib2
import csv

# hwrt modules
import hwrt
import hwrt.utils as utils


# Global variables
preprocessing_queue = None
feature_list = None
model = None
output_semantics = None
n = 10


def submit_recording(raw_data_json):
    """Submit a recording to the database on write-math.com."""
    url = "http://www.martin-thoma.de/write-math/classify/index.php"
    headers = {'User-Agent': 'Mozilla/5.0',
               'Content-Type': 'application/x-www-form-urlencoded'}
    payload = {'drawnJSON': raw_data_json}

    s = requests.Session()
    req = requests.Request('POST', url, headers=headers, data=payload)
    prepared = req.prepare()
    s.send(prepared)


def show_results(results, n=10):
    """Show the TOP n results of a classification."""
    import nntoolkit
    classification = nntoolkit.evaluate.show_results(results, n)
    return "<pre>" + classification.replace("\n", "<br/>") + "</pre>"

# configuration
DEBUG = True

template_path = utils.get_template_folder()

# create our little application :)
app = Flask(__name__, template_folder=template_path)
Bootstrap(app)
app.config.from_object(__name__)


@app.route('/', methods=['POST', 'GET'])
def index():
    """Use heartbeat."""
    return '<a href="interactive">interactive</a>'


@app.route('/interactive', methods=['POST', 'GET'])
def interactive():
    """Interactive classifier."""
    global n
    if request.method == 'GET' and request.args.get('heartbeat', '') != "":
        return request.args.get('heartbeat', '')
    if request.method == 'POST':
        raw_data_json = request.form['drawnJSON']

        # Check recording
        try:
            json.loads(raw_data_json)
        except ValueError:
            return "Invalid JSON string: %s" % raw_data_json

        # Submit recorded json to database
        submit_recording(raw_data_json)

        # Classify
        model_path = pkg_resources.resource_filename('hwrt', 'misc/')
        model = os.path.join(model_path, "model.tar")
        logging.info("Model: %s", model)
        results = utils.evaluate_model_single_recording(model, raw_data_json)

        # Show classification page
        page = show_results(results, n=n)
        page += '<a href="../interactive">back</a>'
        return page
    else:
        # Page where the user can enter a recording
        return render_template('canvas.html')


def get_json_result(results, n=10):
    """Return the top ``n`` results as a json list.
    >>> results = [{'symbolnr': 2, \
                    'probability': 0.65, \
                    'semantics': '\\alpha'}, \
                   {'symbolnr': 45, \
                    'probability': 0.25, \
                    'semantics': '\\propto'}, \
                   {'symbolnr': 15, \
                    'probability': 0.0512, \
                    'semantics': '\\varpropto'}]
    >>> get_json_result(results, n=10)
    [{'\\alpha': 0.65}, {'\\propto': 0.25}, {'\\varpropto': 0.0512}]
    """
    s = []
    for res in results[:min(len(results), n)]:
        s.append({res['semantics']: res['probability']})
    return json.dumps(s)


@app.route('/worker', methods=['POST', 'GET'])
def worker():
    """Implement a worker for write-math.com."""
    global preprocessing_queue, feature_list, model, output_semantics, n
    if request.method == 'POST':
        raw_data_json = request.form['classify']

        # Check recording
        try:
            json.loads(raw_data_json)
        except ValueError:
            return "Invalid JSON string: %s" % raw_data_json

        # Classify
        evaluate = utils.evaluate_model_single_recording_preloaded
        results = evaluate(preprocessing_queue,
                           feature_list,
                           model,
                           output_semantics,
                           raw_data_json)
        return get_json_result(results, n=n)
    else:
        # Page where the user can enter a recording
        return "Classification Worker (Version %s)" % hwrt.__version__


def fix_writemath_answer(results):
    """Bring ``results`` into a format that is accepted by
       write-math.com
    """
    new_results = []
    # Read csv
    translate = {}
    model_path = pkg_resources.resource_filename('hwrt', 'misc/')
    translation_csv = os.path.join(model_path, 'latex2writemathindex.csv')
    with open(translation_csv, 'rb') as csvfile:
        spamreader = csv.reader(csvfile, delimiter=',', quotechar='"')
        for csvrow in spamreader:
            if len(csvrow) == 1:
                writemathid = csvrow[0]
                latex = ""
            else:
                writemathid, latex = csvrow
            translate[latex] = writemathid

    for i, el in enumerate(results):
        # el = {'symbolnr': 214,
        #       'semantics': '\\triangleq',
        #       'probability': 0.030137155301477141}
        new_results.append({'symbolnr': el['symbolnr'],
                            'semantics': translate[el['semantics']],
                            'probability': el['probability']})
        if i >= 10 or (i > 0 and el['probability'] < 0.20):
            break
    return new_results


@app.route('/work', methods=['POST', 'GET'])
def work():
    """Implement a worker for write-math.com."""
    global preprocessing_queue, feature_list, model, output_semantics, n
    print("Start working!")
    for i in range(1000):
        # contact the write-math server and get something to classify
        url = "http://www.martin-thoma.de/write-math/get_unclassified/"
        response = urllib2.urlopen(url)
        page_source = response.read()
        parsed_json = json.loads(page_source)
        raw_data_json = parsed_json['recording']

        # Classify
        # Check recording
        try:
            json.loads(raw_data_json)
        except ValueError:
            return "Raw Data ID %s; Invalid JSON string: %s" % (parsed_json['id'], raw_data_json)

        print("http://www.martin-thoma.de/write-math/view/?raw_data_id=%s" %
              str(parsed_json['id']))

        # Classify
        evaluate = utils.evaluate_model_single_recording_preloaded
        results = evaluate(preprocessing_queue,
                           feature_list,
                           model,
                           output_semantics,
                           raw_data_json)

        # TODO: Submit classification
        results = fix_writemath_answer(results)
        results_json = get_json_result(results, n=n)
        headers = {'User-Agent': 'Mozilla/5.0',
                   'Content-Type': 'application/x-www-form-urlencoded'}
        payload = {'recording_id': parsed_json['id'],
                   'results': results_json}

        s = requests.Session()
        req = requests.Request('POST', url, headers=headers, data=payload)
        prepared = req.prepare()
        resp = s.send(prepared)

        # print("Raw Data ID: %s", str(parsed_json['id']))
        # print("Answer: %s", str(results_json))
        # print("Resp: %s" % resp.text)
    print("Done")
    return "Done"


def get_parser():
    """Return the parser object for this script."""
    from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
    parser = ArgumentParser(description=__doc__,
                            formatter_class=ArgumentDefaultsHelpFormatter)
    parser.add_argument("-n",
                        dest="n", default=10, type=int,
                        help="Show TOP-N results")
    return parser


def main():
    """Main function starting the webserver."""
    global preprocessing_queue, feature_list, model, output_semantics, n
    logging.info("Start reading model...")
    model_path = pkg_resources.resource_filename('hwrt', 'misc/')
    model_file = os.path.join(model_path, "model.tar")
    (preprocessing_queue, feature_list, model,
     output_semantics) = utils.load_model(model_file)
    logging.info("Start webserver...")
    app.run()

if __name__ == '__main__':
    n = get_parser().parse_args().n
    main()



#!/usr/bin/env python

"""
Add `is_in_testset` to raw_datasets in MySQL database, so that at least 10%
of the data online has the flag `is_in_testset`.
"""

import pymysql
import pymysql.cursors
import random
import math

# hwrt modules
from hwrt.HandwrittenData import HandwrittenData
import hwrt.utils as utils


def main(mysql):
    """Add testset flag to recordings in MySQL database."""
    connection = pymysql.connect(host=mysql['host'],
                                 user=mysql['user'],
                                 passwd=mysql['passwd'],
                                 db=mysql['db'],
                                 cursorclass=pymysql.cursors.DictCursor)
    cursor = connection.cursor()

    # Download all datasets
    sql = ("SELECT `id`, `formula_in_latex` FROM `wm_formula` "
           "WHERE `is_important` = 1 ORDER BY `id` ASC")
    cursor.execute(sql)
    datasets = cursor.fetchall()

    for i, data in enumerate(datasets):
        fid, formula_in_latex = data['id'], data['formula_in_latex']
        print("%i: Create testset for %s (id: %i)..." % (i,
                                                         formula_in_latex,
                                                         fid))
        sql = ("SELECT `id`, `is_in_testset` FROM `wm_raw_draw_data` "
               "WHERE `accepted_formula_id` = %i" % fid)
        cursor.execute(sql)
        raw_datasets = cursor.fetchall()
        is_in_testset = 0
        raw_candidate_ids = []
        for j, raw_data in enumerate(raw_datasets):
            if raw_data['is_in_testset'] == 1:
                is_in_testset += 1
            else:
                raw_candidate_ids.append(raw_data['id'])
        testset_ratio = 0.1
        testset_total = int(math.ceil(len(raw_datasets)*testset_ratio))
        remaining = testset_total - is_in_testset

        if remaining > 0:
            print("%i in testset. Add remaining %i datasets to testset..." %
                  (is_in_testset, remaining))
            add_new = random.sample(raw_candidate_ids, remaining)
            if len(add_new) < 20:
                for el in add_new:
                    print("http://write-math.com/view/?raw_data_id=%i" % el)
            for rid in add_new:
                sql = ("UPDATE `wm_raw_draw_data` SET `is_in_testset`=1 "
                       "WHERE `id` = %i LIMIT 1") % rid
                cursor.execute(sql)
            connection.commit()


def get_parser():
    """Return the parser object for this script."""
    from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
    parser = ArgumentParser(description=__doc__,
                            formatter_class=ArgumentDefaultsHelpFormatter)
    return parser


if __name__ == '__main__':
    args = get_parser().parse_args()
    cfg = utils.get_database_configuration()
    if 'mysql_online' in cfg:
        main(cfg['mysql_online'])
    if 'mysql_local' in cfg:
        main(cfg['mysql_local'])



#!/usr/bin/env python

"""Check if all necessary modules / programs / files for HWRT are there and
   if the version is ok.
"""

import imp
import sys
import platform
import os
import pkg_resources

# hwrt
from . import utils


class bcolors(object):
    """Terminal colors with ANSI escape codes."""
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'


def which(program):
    """Get the path of a program or ``None`` if ``program`` is not in path."""
    def is_exe(fpath):
        """Check for windows users."""
        return os.path.isfile(fpath) and os.access(fpath, os.X_OK)

    fpath, fname = os.path.split(program)
    if fpath:
        if is_exe(program):
            return program
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            path = path.strip('"')
            exe_file = os.path.join(path, program)
            if is_exe(exe_file):
                return exe_file

    return None


def check_python_version():
    """Check if the currently running Python version is new enough."""
    # Required due to multiple with statements on one line
    req_version = (2, 7)
    cur_version = sys.version_info
    if cur_version >= req_version:
        print("Python version... %sOK%s (found %s, requires %s)" %
              (bcolors.OKGREEN, bcolors.ENDC, str(platform.python_version()),
               str(req_version[0]) + "." + str(req_version[1])))
    else:
        print("Python version... %sFAIL%s (found %s, requires %s)" %
              (bcolors.FAIL, bcolors.ENDC, str(cur_version),
               str(req_version)))


def check_python_modules():
    """Check if all necessary / recommended modules are installed."""
    print("\033[1mCheck modules\033[0m")
    required_modules = ['argparse', 'matplotlib', 'natsort', 'pymysql',
                        'cPickle', 'theano', 'dropbox', 'yaml',
                        'webbrowser', 'hashlib', 'numpy',
                        'jinja2', 'Tkinter']
    found = []
    for required_module in required_modules:
        try:
            imp.find_module(required_module)
            check = "module '%s' ... %sfound%s" % (required_module,
                                                   bcolors.OKGREEN,
                                                   bcolors.ENDC)
            print(check)
            found.append(required_module)
        except ImportError:
            print("module '%s' ... %sNOT%s found" % (required_module,
                                                     bcolors.WARNING,
                                                     bcolors.ENDC))

    if "argparse" in found:
        import argparse
        print("argparse version: %s (1.1 tested)" % argparse.__version__)
    if "matplotlib" in found:
        import matplotlib
        print("matplotlib version: %s (1.2.1 tested)" % matplotlib.__version__)
    if "natsort" in found:
        import natsort
        print("natsort version: %s (3.4.0 tested, 3.4.0 > required)" %
              natsort.__version__)
    if "pymysql" in found:
        import pymysql
        print("pymysql version: %s (0.6.3 tested)" %
              pymysql.__version__)
    if "theano" in found:
        import theano
        print("theano version: %s (0.6.0 tested)" %
              theano.__version__)
    if "numpy" in found:
        import numpy
        print("numpy version: %s (1.8.1 tested)" %
              numpy.__version__)
    if "yaml" in found:
        import yaml
        print("yaml version: %s (3.11 tested)" %
              yaml.__version__)
    if "jinja2" in found:
        import jinja2
        print("jinja2 version: %s (2.7.3 tested)" %
              jinja2.__version__)
    if "cPickle" in found:
        import cPickle
        print("cPickle version: %s (1.71 tested)" %
              cPickle.__version__)
        print("cPickle HIGHEST_PROTOCOL: %s (2 required)" %
              cPickle.HIGHEST_PROTOCOL)


def check_executables():
    """Check if all necessary / recommended executables are installed."""
    print("\033[1mCheck executables\033[0m")
    required_executables = ["pfile_create", utils.get_nntoolkit()]
    for executable in required_executables:
        path = which(executable)
        if path is None:
            print("%s ... %sNOT%s found" % (executable, bcolors.WARNING,
                                            bcolors.ENDC))
            print("Try 'http://martin-thoma.com/what-are-pfiles/' for "
                  "instructions how to get it.")
        else:
            print("%s ... %sfound%s at %s" % (executable, bcolors.OKGREEN,
                                              bcolors.ENDC, path))


def main():
    """Execute all checks."""
    check_python_version()
    check_python_modules()
    check_executables()
    home = os.path.expanduser("~")
    print("\033[1mCheck files\033[0m")
    rcfile = os.path.join(home, ".hwrtrc")
    if os.path.isfile(rcfile):
        print("~/.hwrtrc... %sFOUND%s" %
              (bcolors.OKGREEN, bcolors.ENDC))
    else:
        print("~/.hwrtrc... %sNOT FOUND%s" %
              (bcolors.FAIL, bcolors.ENDC))
    misc_path = pkg_resources.resource_filename('hwrt', 'misc/')
    print("misc-path: %s" % misc_path)


if __name__ == '__main__':
    main()



#!/usr/bin/env python

"""Calculate the distance between line segments."""

import math
import itertools


class Point(object):
    """A two dimensional point."""
    def __init__(self, x, y):
        self.x = float(x)
        self.y = float(y)

    def dist_to(self, p2):
        """Measure the distance to another point."""
        return math.hypot(self.x - p2.x, self.y - p2.y)

    def __eq__(self, other):
        return (self.x-other.x)**2 + (self.y-other.y)**2 < 0.000001

    def __hash__(self):
        return hash((self.x, self.y))

    def __repr__(self):
        return "p(%0.2f, %0.2f)" % (self.x, self.y)

    def __str__(self):
        return self.__repr__()


class LineSegment(object):
    """A line segment in a two dimensional space."""
    def __init__(self, p1, p2):
        assert isinstance(p1, Point), \
            "p1 is not of type Point, but of %r" % type(p1)
        assert isinstance(p2, Point), \
            "p2 is not of type Point, but of %r" % type(p2)
        self.p1 = p1
        self.p2 = p2

    def dist_to(self, l2):
        """Measure the distance to another line segment."""
        return segments_distance(self, l2)

    def get_slope(self):
        """Return the slope m of this line segment."""
        # y1 = m*x1 + t
        # y2 = m*x2 + t => y1-y2 = m*(x1-x2) <=> m = (y1-y2)/(x1-x2)
        return ((self.p1.y-self.p2.y) / (self.p1.x-self.p2.x))

    def get_offset(self):
        """Get the offset t of this line segment."""
        return self.p1.y-self.get_slope()*self.p1.x

    def __repr__(self):
        return "line[%s -> %s]" % (str(self.p1), str(self.p2))

    def __str__(self):
        return self.__repr__()


class PolygonalChain(object):
    """A list of line segments."""
    def __init__(self, pointlist):
        assert isinstance(pointlist, list), \
            "lineSegments is not of type list, but of %r" % type(pointlist)
        self.lineSegments = []
        for point1, point2 in zip(pointlist, pointlist[1:]):
            point1 = Point(point1['x'], point1['y'])
            point2 = Point(point2['x'], point2['y'])
            self.lineSegments.append(LineSegment(point1, point2))

    def __iter__(self):
        return iter(self.lineSegments)

    def __eq__(self, other):
        return self.__dict__ == other.__dict__

    def count_selfintersections(self):
        """ Get the number of self-intersections of this polygonal chain."""
        # This can be solved more efficiently with sweep line
        counter = 0
        for i, j in itertools.combinations(range(len(self.lineSegments)), 2):
            inters = get_segments_intersections(self.lineSegments[i],
                                                self.lineSegments[j])
            if abs(i-j) > 1 and len(inters) > 0:
                counter += 1
        return counter

    def count_intersections(self, lineSegmentsB):
        """Count the intersections of two strokes with each other."""
        lineSegmentsA = self.lineSegments

        # Calculate intersections
        intersection_points = []
        for line1, line2 in itertools.product(lineSegmentsA, lineSegmentsB):
            intersection_points += get_segments_intersections(line1, line2)
        return len(set(intersection_points))


class BoundingBox(object):
    """A rectangle whichs sides are in parallel to the axes."""
    def __init__(self, p1, p2):
        assert isinstance(p1, Point), \
            "p1 is not of type Point, but of %r" % type(p1)
        assert isinstance(p2, Point), \
            "p2 is not of type Point, but of %r" % type(p2)
        assert p1.x <= p2.x, "p1.x <= p2.x (%0.2f and %0.2f)" % (p1.x, p2.x)
        assert p1.y <= p2.y, "p1.y <= p2.y (%0.2f and %0.2f)" % (p1.y, p2.y)
        self.p1 = p1
        self.p2 = p2

    def __repr__(self):
        return "BoundingBox[%s, %s]" % (str(self.p1), str(self.p2))

    def __str__(self):
        return self.__repr__()

    def get_area(self):
        """Calculate area of bounding box."""
        return (self.p2.x-self.p1.x)*(self.p2.y-self.p1.y)

    def get_width(self):
        """
        >>> BoundingBox(Point(0,0), Point(3,2)).get_width()
        3.0
        """
        return self.p2.x - self.p1.x

    def get_height(self):
        """
        >>> BoundingBox(Point(0,0), Point(3,2)).get_height()
        2.0
        """
        return self.p2.y - self.p1.y

    def get_largest_dimension(self):
        """Get the larger dimension of the bounding box."""
        return max(self.get_height(), self.get_width())


def do_bb_intersect(a, b):
    """Check if BoundingBox a intersects with BoundingBox b."""
    return a.p1.x <= b.p2.x \
        and a.p2.x >= b.p1.x \
        and a.p1.y <= b.p2.y \
        and a.p2.y >= b.p1.y


def segments_distance(segment1, segment2):
    """Calculate the distance between two line segments in the plane.

    >>> a = LineSegment(Point(1,0), Point(2,0))
    >>> b = LineSegment(Point(0,1), Point(0,2))
    >>> "%0.2f" % segments_distance(a, b)
    '1.41'
    >>> c = LineSegment(Point(0,0), Point(5,5))
    >>> d = LineSegment(Point(2,2), Point(4,4))
    >>> e = LineSegment(Point(2,2), Point(7,7))
    >>> "%0.2f" % segments_distance(c, d)
    '0.00'
    >>> "%0.2f" % segments_distance(c, e)
    '0.00'
    """
    if len(get_segments_intersections(segment1, segment2)) >= 1:
        return 0
    # try each of the 4 vertices w/the other segment
    distances = []
    distances.append(point_segment_distance(segment1.p1, segment2))
    distances.append(point_segment_distance(segment1.p2, segment2))
    distances.append(point_segment_distance(segment2.p1, segment1))
    distances.append(point_segment_distance(segment2.p2, segment1))
    return min(distances)


def get_segments_intersections(segment1, segment2):
    """Return at least one point in a list where segments intersect if an
       intersection exists. Otherwise, return an empty list.
    >>> get_segments_intersections(LineSegment(Point(0,0), Point(1,0)), \
                                   LineSegment(Point(0,0), Point(1,0)))
    [Point(0,0)]
    """
    dx1 = segment1.p2.x - segment1.p1.x
    dy1 = segment1.p2.y - segment1.p1.y
    dx2 = segment2.p2.x - segment2.p1.x
    dy2 = segment2.p2.y - segment2.p1.y
    delta = dx2 * dy1 - dy2 * dx1
    if delta == 0:  # parallel segments
        # Line segments could be (partially) identical.
        # In that case this functin should return True.
        if dx1 == 0:
            # Lines segments are vertical
            if segment1.p1.x == segment2.p1.x:
                if segment1.p1.y > segment1.p2.y:
                    segment1.p1, segment1.p2 = segment1.p2, segment1.p1
                if segment2.p1.y > segment2.p2.y:
                    segment2.p1, segment2.p2 = segment2.p2, segment2.p1
                # Lines segments are on the same line
                if segment1.p1.y <= segment2.p1.y <= segment1.p2.y:
                    return [Point(segment1.p1.x, segment2.p1.y)]
                if segment2.p1.y <= segment1.p1.y <= segment2.p2.y:
                    return [Point(segment1.p1.x, segment1.p1.y)]
        else:
            # The equation f(x) = m*x + t defines any non-vertical line
            t1 = segment1.get_offset()
            t2 = segment2.get_offset()
            if t1 == t2:  # line segments are on the same line
                if segment1.p1.x <= segment2.p1.x <= segment1.p2.x:
                    return [Point(segment2.p1.x,
                                  segment2.get_slope()*segment2.p1.x+t2)]
                if segment2.p1.x <= segment1.p1.x <= segment2.p2.x:
                    return [Point(segment1.p1.x,
                                  segment1.get_slope()*segment1.p1.x+t1)]
        return []
    if dx2 == 0:  # Line 2 is a vertical line, but line 1 isn't
        segment1, segment2 = segment2, segment1
        dx1, dx2 = dx2, dx1
    if dx1 == 0:  # Line 1 is a vertical line, but line 2 isn't
        if segment2.p1.x > segment2.p2.x:
            segment2.p1, segment2.p2 = segment2.p2, segment2.p1
        if segment2.p1.x <= segment1.p1.x <= segment2.p2.x:
            # The x-values overlap
            m2 = segment2.get_slope()
            t2 = segment2.get_offset()
            y = m2*segment1.p1.x + t2
            if segment1.p1.y > segment1.p2.y:
                segment1.p1, segment1.p2 = segment1.p2, segment1.p1
            if segment1.p1.y <= y <= segment1.p2.y:
                return [Point(segment1.p1.x, y)]
            else:
                return []
        else:
            return []

    m1, t1 = segment1.get_slope(), segment1.get_offset()
    m2, t2 = segment2.get_slope(), segment2.get_offset()
    x = (t2-t1)/(m1-m2)
    if segment1.p1.x > segment1.p2.x:
        segment1.p1, segment1.p2 = segment1.p2, segment1.p1
    if segment2.p1.x > segment2.p2.x:
        segment2.p1, segment2.p2 = segment2.p2, segment2.p1
    if (segment1.p1.x <= x <= segment1.p2.x) and \
       (segment2.p1.x <= x <= segment2.p2.x):
        # The intersection is on both line segments - not only on the lines
        return [Point(x, m1*x+t1)]
    else:
        return []


def point_segment_distance(point, segment):
    """
    >>> a = LineSegment(Point(1,0), Point(2,0))
    >>> b = LineSegment(Point(2,0), Point(0,2))
    >>> point_segment_distance(Point(0,0), a)
    1.0
    >>> "%0.2f" % point_segment_distance(Point(0,0), b)
    '1.41'
    """
    assert isinstance(point, Point), \
        "point is not of type Point, but of %r" % type(point)
    dx = segment.p2.x - segment.p1.x
    dy = segment.p2.y - segment.p1.y
    if dx == dy == 0:  # the segment's just a point
        return point.dist_to(segment.p1)

    if dx == 0:  # It's a straight vertical line
        pIsBelowP1 = point.y <= segment.p1.y and segment.p1.y <= segment.p2.y
        pIsBelowP2 = point.y <= segment.p2.y and segment.p2.y <= segment.p1.y
        pIsAboveP2 = segment.p1.y <= segment.p2.y and segment.p2.y <= point.y
        pIsAboveP1 = segment.p2.y <= segment.p1.y and segment.p1.y <= point.y
        if pIsBelowP1 or pIsAboveP1:
            return point.dist_to(segment.p1)
        elif pIsBelowP2 or pIsAboveP2:
            return point.dist_to(segment.p2)

    if dy == 0:  # It's a straight horizontal line
        pIsLeftP1 = point.x <= segment.p1.x and segment.p1.x <= segment.p2.x
        pIsLeftP2 = point.x <= segment.p2.x and segment.p2.x <= segment.p1.x
        pIsRightP2 = segment.p1.x <= segment.p2.x and segment.p2.x <= point.x
        pIsRightP1 = segment.p2.x <= segment.p1.x and segment.p1.x <= point.x
        if pIsLeftP1 or pIsRightP1:
            return point.dist_to(segment.p1)
        elif pIsLeftP2 or pIsRightP2:
            return point.dist_to(segment.p2)

    # Calculate the t that minimizes the distance.
    t = ((point.x - segment.p1.x) * dx + (point.y - segment.p1.y) * dy) / \
        (dx * dx + dy * dy)

    # See if this represents one of the segment's
    # end points or a point in the middle.
    if t < 0:
        dx = point.x - segment.p1.x
        dy = point.y - segment.p1.y
    elif t > 1:
        dx = point.x - segment.p2.x
        dy = point.y - segment.p2.y
    else:
        near_x = segment.p1.x + t * dx
        near_y = segment.p1.y + t * dy
        dx = point.x - near_x
        dy = point.y - near_y
    return math.hypot(dx, dy)


def perpendicular_distance(p3, p1, p2):
    """
    Calculate the distance from p3 to the stroke defined by p1 and p2.
    The distance is the length of the perpendicular from p3 on p1.
    :param p1: start of stroke
    :type p1: dictionary with "x" and "y"
    :param p2: end of stroke
    :type p2: dictionary with "x" and "y"
    :param p3: point
    :type p3: dictionary with "x" and "y"
    """
    px = p2['x']-p1['x']
    py = p2['y']-p1['y']

    squared_distance = px*px + py*py
    if squared_distance == 0:
        # The line is in fact only a single dot.
        # In this case the distance of two points has to be
        # calculated
        linePoint = Point(p1['x'], p1['y'])
        point = Point(p3['x'], p3['y'])
        return linePoint.dist_to(point)

    u = ((p3['x'] - p1['x'])*px + (p3['y'] - p1['y'])*py) / squared_distance

    if u > 1:
        u = 1
    elif u < 0:
        u = 0

    x = p1['x'] + u * px
    y = p1['y'] + u * py

    dx = x - p3['x']
    dy = y - p3['y']

    # Note: If the actual distance does not matter,
    # if you only want to compare what this function
    # returns to other results of this function, you
    # can just return the squared distance instead
    # (i.e. remove the sqrt) to gain a little performance

    dist = math.sqrt(dx*dx + dy*dy)
    return dist


if __name__ == '__main__':
    import doctest
    doctest.testmod()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Data multiplication algorithms.

Each algorithm works on the HandwrittenData class. They have to be applied like
this:

 >>> import data_multiplication as multiply
 >>> a = HandwrittenData(...)
 >>> multiplication_queue = [multiply.copy(10),
                            mulitply.rotate(-30, 30, 5)
                            ]
 >>> x = a.multiply(multiplication_queue)

"""

import numpy
import sys
import math
from copy import deepcopy

# hwrt modules
from hwrt import HandwrittenData
from hwrt import utils


def get_data_multiplication_queue(model_description_multiply):
    """Get features from a list of dictionaries

    >>> l = [{'Multiply': [{'nr': 1}]}, \
             {'Rotate': [{'minimum':-30}, {'maximum': 30}, {'step': 5}]}]
    >>> get_data_multiplication_queue(l)
    [Multiply (1 times), Rotate (-30.00, 30.00, 5.00)]
    """
    return utils.get_objectlist(model_description_multiply,
                                config_key='data_multiplication',
                                module=sys.modules[__name__])

# Only data multiplication classes follow
# Everyone must have a __str__, __repr__, __call__ and get_dimension function
# where
# * __call__ must take exactly one argument of type HandwrittenData
# * __call__ must return a list of HandwrittenData objects


# Local features


class Multiply(object):

    """Copy the data n times."""

    def __init__(self, nr=1):
        self.nr = nr

    def __repr__(self):
        return "Multiply (%i times)" % self.nr

    def __str__(self):
        return self.__repr__()

    def __call__(self, handwritten_data):
        assert isinstance(handwritten_data, HandwrittenData.HandwrittenData), \
            "handwritten data is not of type HandwrittenData, but of %r" % \
            type(handwritten_data)
        new_trainging_set = []
        for _ in range(self.nr):
            new_trainging_set.append(handwritten_data)
        training_set = new_trainging_set
        return training_set


class Rotate(object):

    """Add rotational variants of the recording."""

    def __init__(self, minimum=-30.0, maximum=30.0, num=5):
        self.min = minimum
        self.max = maximum
        self.num = num

    def __repr__(self):
        return "Rotate (%0.2f, %0.2f, %0.2f)" % (self.min, self.max, self.num)

    def __str__(self):
        return self.__repr__()

    def __call__(self, handwritten_data):
        assert isinstance(handwritten_data, HandwrittenData.HandwrittenData), \
            "handwritten data is not of type HandwrittenData, but of %r" % \
            type(handwritten_data)
        new_trainging_set = []
        xc, yc = handwritten_data.get_center_of_mass()
        pointlist = handwritten_data.get_pointlist()
        for rotation in numpy.linspace(self.min, self.max, self.num):
            new_poinlist = []
            # Rotate pointlist around center of mass (xc, yc)
            for line in pointlist:
                new_line = []
                for point in line:
                    # Calculate rotation
                    # xnew, ynew = xc, yc
                    # (xnew, ynew) += (x-xc, y-yc)* (cos(rot.)  -sin(rot.))
                    #                               (sin(rot.)   cos(rot.))
                    x, y = point['x'], point['y']
                    cos = math.cos(math.radians(rotation))
                    sin = math.sin(math.radians(rotation))
                    xnew = cos*(x-xc) - sin*(y-yc) + xc
                    ynew = sin*(x-xc) + cos*(y-yc) + yc
                    new_line.append({'x': xnew,
                                     'y': ynew,
                                     'time': point['time']})
                new_poinlist.append(new_line)
            #create the new handwritten data object
            hwd_tmp = deepcopy(handwritten_data)
            hwd_tmp.set_pointlist(new_poinlist)
            new_trainging_set.append(hwd_tmp)
        training_set = new_trainging_set
        return training_set

if __name__ == '__main__':
    import doctest
    doctest.testmod()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Features in development."""

import urllib
import os

# hwrt modules
import hwrt.HandwrittenData as HandwrittenData
import hwrt.utils as utils


class Bitmap(object):

    """n × n grayscale bitmap of the recording."""

    normalize = True

    def __init__(self, n=28):
        self.n = n  # Size of the bitmap (n x n)

    def __repr__(self):
        return ("Bitmap (n=%i)\n") % (self.n)

    def __str__(self):
        return self.__repr__()

    def get_dimension(self):
        """Get the dimension of the returned feature. This equals the number
           of elements in the returned list of numbers."""
        return self.n**2

    def __call__(self, handwritten_data):
        assert isinstance(handwritten_data, HandwrittenData.HandwrittenData), \
            "handwritten data is not of type HandwrittenData, but of %r" % \
            type(handwritten_data)
        x = []
        url = "http://localhost/write-math/website/raw-data/"
        raw_data_id = handwritten_data.raw_data_id
        project_root = utils.get_project_root()
        foldername = os.path.jon(project_root, "bitmaps")
        f = urllib.urlopen("{url}{id}.svg".format(url=url, id=raw_data_id))
        with open("%s%i.svg" % (foldername, raw_data_id), "wb") as imgFile:
            imgFile.write(f.read())

        command = ("convert -size 28x28 {folder}{id}.svg  -resize {n}x{n} "
                   "-gravity center -extent {n}x{n} "
                   "-monochrome {folder}{id}.png").format(id=raw_data_id,
                                                          n=self.n,
                                                          # url=url,
                                                          folder=foldername)
        os.system(command)
        from PIL import Image
        im = Image.open("%s%i.png" % (foldername, raw_data_id))
        pix = im.load()
        # pixel_image = [[0 for i in range(28)] for j in range(28)]
        for i in range(28):
            for j in range(28):
                # pixel_image[i][j] = pix[i, j]
                x.append(pix[i, j])
        assert self.get_dimension() == len(x), \
            "Dimension of %s should be %i, but was %i" % \
            (self.__str__(), self.get_dimension(), len(x))
        return x
